
// merlin documentation app :: written with merlin.

var app = merlin ()

app.use(function (state, emitter) {
  state.demo = 0
  emitter.on('demo:wahoo', console.log)
})

app.route('/', home)
app.route('/documentation', documentation)
app.route('/examples', examples)
app.route('/contact', contact)
app.route('/faq/:id', faq)
app.route('/faq', faq)

app.mount()

function header () {
	return `<nav class="pa3 pa4-ns">
	  <a class="link dim black b f1 f-headline-ns tc db mb3 mb4-ns" href="#/" title="Home">Merlin</a>
	  <div class="tc pb3">
		<a class="link dim gray f6 f5-ns dib mr3" href="#/" title="Home">Home</a>
		<a class="link dim gray f6 f5-ns dib mr3" href="/#/documentation" title="Documentation">Documentation</a>
		<a class="link dim gray f6 f5-ns dib mr3" href="#/examples" title="Examples">Examples</a>
		<a class="link dim gray f6 f5-ns dib mr3" href="#/faq" title="faq">FAQ</a>
		<a class="link dim gray f6 f5-ns dib" href="#/contact" title="Contact">Contact</a>
	  </div>
	</nav>`;
}

function footer () {
	return `<footer class="bg-near-black white-80 pv5 pv6-l ph4">
	  <p class="f6"><span class="dib mr4 mr5-ns">2020 Lloyds banking group.</span>
		<a class="link white-80 hover-light-purple" href="https://confluence.devops.lloydsbanking.com/display/CLOVA/CLOVA">Web Analytics and Tag Management</a> /
		<a class="link white-80 hover-green" href="https://confluence.devops.lloydsbanking.com/x/Ewz6GQ">author @m-onz</a>
	  </p>
	</footer>`;
}

function escapeHTML(str){
	return new Option(str).innerHTML;
}

function home (state, emitter) {
  return `
	${header()}
	<section class="mw5 mw7-ns center bg-light-gray pa3 ph5-ns">
	  <h1 class="mt0">Merlin :: js library</h1>
	  <p class="lh-copy measure">
		Merlin is javascript library for creating single page apps.
		<pre>
			var app = merlin ()

			app.use(function (state, emitter) {
			  state.demo = 0;
			  emitter.on('demo:wahoo', console.log)	
			})
			
			app.route('/', view)
			app.mount()
			
			function view () {
				return \`${escapeHTML("<h1>hello world!</h1>")}\`
			}
		</pre>
	  </p>
	</section>
	${footer()}
  `;
}

function documentation (state, emitter) {	


  /*
	documentation::
	
	library methods & parameters
		
	how to use it:
	
		add the dependency:
		
			add merlin bundle via script tag or inline.
	
		middleware
			.use
		
		routes
		
		state & events
		
			the state object
			the emitter
			
		
		anatomy of the program & components
		
			views, templates and composition
		
		state.ready
		state.form
  
  */


  return `
	${header()}
	<section class="mw5 mw7-ns center bg-light-gray pa3 ph5-ns">
	  <h1 class="mt0">Merlin :: documentation</h1>
	  <p class="lh-copy measure">
		Merlin is javascript library for creating single page apps.
		<pre>
			var app = merlin ()
		</pre>
	  </p>
	</section>
	${footer()}
  `;
}

function examples (state, emitter) {

  state.ready(() => {
	state.form("form", function (data, e) {
	  var log = document.getElementById('log');
	  log.textContent = `Form Submitted! Time stamp: ${e.timeStamp}`;
	  console.log("test", data.get("test"))
	  console.log("demo", data.get("demo"))
	})
  })

  return `
	${header()}
	<section class="mw5 mw7-ns center bg-light-gray pa3 ph5-ns">
		<h1 class="mt0">Merlin :: examples</h1>
		<p class="lh-copy measure">
			Form post example ::
			<pre>
				var app = merlin ()
			</pre>
		</p>

		<article class="pa4 black-80">
			<form id="form">
			  <label>Test field: <input type="text" name="test"></label>
			  <label>Test field 2: <input type="text" name="demo" id="demo" value="hello"></label>
			  <button type="submit">Submit form</button>
			</form>
			<p id="log"></p>
		</article>

	</section>
	${footer()}
  `;
}

function contact (state, emitter) {	
  return `
	<div>
		${header()}
		<section class="mw5 mw7-ns center bg-light-gray pa3 ph5-ns">
			<h1 class="mt0">Merlin :: js library</h1>
			<p class="lh-copy measure">
				contact @m-onz about issues, problems and help.
			</p>
			<a href="#/faq/2">faq page 2</a>
		</section>
		${footer()}
	</div>
	`;
}

function faq (state, emitter) {	

  console.log(':: test route parameters :: ', state.params, typeof state.params, ' :: ')

  return `
	<div>
		${header()}
		<section class="mw5 mw7-ns center bg-light-gray pa3 ph5-ns">
			<h1 class="mt0">Merlin :: faq</h1>
			<p class="lh-copy measure">
				Example of route parameters
			</p>
		</section>
		${footer()}
	</div>
	`;
}
