﻿
/*

  merlin
  create single page apps with minimal dependencies::
  
	current spec:
		single page app library using hash routing
		works via script tags with no extra bundling or compilation
		hooks onto the window object (can be modified dynamically)
		very small code base suitable for use as a confluence HTML macro
		needs "modern js" to work: template strings, spread operator, arrow functions.
		works in edge but no ie.
  
  @m-onz
  
  create an NPM package...
  
  then use unkpkg to include the script in pages...
  
  
 
*/

var assert = {
	ok: function (input, message) {
		if (!input) throw Error((message) ? message : 'An error has occured')
	},
	equal: function (a, b, message) {
		if (a !== b) throw Error((message) ? message : 'An error has occured')
	}
}

function merlin () {
  if (! (this instanceof merlin)) return new merlin ()
  this.middleware = []
  this.routes = {}
  this.state = {}
  this.params = {}
  var self = this
  this.emitter = Nanobus()
  this.router = Nanorouter()
  setTimeout(() => {
	this.app = document.getElementById("app")
  })
  window.onhashchange = function() {
	var current_route = location
		.hash.replace(/\#/g, '') || '/#';
	// <a href="#!" will be ignored...
	if (!current_route.includes('!')) self.router.emit(current_route)
  }
}

merlin.prototype._finish_setup = function () {
  var self = this 
  Object.keys(self.routes).forEach(function (route) {
    self.router.on(route, function (params) {
		self.state.params = params
		self.app.innerHTML = self.routes[route](self.state, self.emitter)
	})
  })
}

merlin.prototype.use = function (middleware) {
  assert.ok(typeof middleware === 'function')
  this.middleware.push(middleware)
}

merlin.prototype.route = function (route, handler) {
  assert.ok(typeof route === 'string')
  assert.ok(typeof handler === 'function')  
  this.routes[route] = handler;
}

merlin.prototype.mount = function (element) {
  var self = this;
  var default_middleware = function (state, emit) {
    state.ready = function (...args) { return setTimeout(...args); }
    state.form = function addForm (id, callback) {
	  var form = document.getElementById(id)
	  form.addEventListener('submit', function (e) {
		e.preventDefault()
	    var data = new FormData(e.currentTarget)
	    callback(data, e)
	  })
    }
  }
  default_middleware(self.state, self.emitter)
  self.middleware.forEach(function (middleware) {
    middleware(self.state, self.emitter)
  })
  self._finish_setup()
  setTimeout(function () {
	self.app.innerHTML = self.routes[
		Object.keys(self.routes)[0]
	](self.state, self.emitter)
  }, 1000)
}

window.merlin = merlin;
